package assets;

import org.lwjgl.util.vector.Vector3f;

import datasets.Projection;
import models.Model;
import renderer.Loader;
import utilities.MNMatrix;

public class AxisVector {
	
	private int dimension;
	private int numberOfDimensions;
	private MNMatrix vec;
	private Model model;
	
	public AxisVector(int dimension, int numberOfDimensions, Loader loader) {
		this.dimension = dimension;
		this.numberOfDimensions = numberOfDimensions;
		vec = new MNMatrix(numberOfDimensions, 1);
		vec.fillEmptyMatrix();
		float[] col = new float[9*3];
        
		vec.setValue(dimension-1, 0, 1f);
		if (this.dimension == 1){
			for (int c = 0; c < 9*3; c+=3){ col[c] = 1f; col[c+1] = 0f; col[c+2] = 0f;}
		} else if (this.dimension == 2){
			for (int c = 0; c < 9*3; c+=3){ col[c] = 1f; col[c+1] = 1f; col[c+2] = 0f;}
		} else if (this.dimension == 3){
			for (int c = 0; c < 9*3; c+=3){ col[c] = 0f; col[c+1] = 0f; col[c+2] = 1f;}
		} 
		
		int[] ind = {0,2,1,1,2,3,4,5,6}; 
		float[] pos = {	0f,-0.005f,0f,
						0f,0.005f,0f,
						0.45f,-0.005f,0,
						0.45f, 0.005f,0,
						0.5f,0,0,
						0.35f,0.03f,0,
						0.35f,-0.03f,0
						};
		model = loader.loadToVAO(pos, ind , col);
	}

	public AxisVector(AxisVector uv) {
		this.dimension = uv.dimension;
		this.numberOfDimensions = uv.numberOfDimensions;
		this.vec = uv.vec;
		this.model = uv.model;
	}

	public Model getModel() {
		return model;
	}

	public MNMatrix getVec() {
		return vec;
	}
	
	public void setVec(MNMatrix vec) {
		this.vec = new MNMatrix(vec.getRows(),vec.getCols());
		this.vec.fillEmptyMatrix();
		for (int r = 0; r < vec.getRows(); r++) {
			for (int c = 0; c < vec.getCols(); c++) {
				this.vec.setValue(r, c, vec.getValue(r, c));
			}
		}
	}

	public float getRotX() {
		return (float)(Boolean.compare(this.dimension > 3, false)*(-Projection.passAngles()[0]))
				+ Projection.passAngles()[0];
	}
	public float getRotY() {
		return (float)(Boolean.compare(this.dimension > 3, false)*(-Projection.passAngles()[1]))
				- (float)(Boolean.compare(this.dimension == 3, false)*(Math.PI)/2.0)+Projection.passAngles()[1];
	}
	public float getRotZ() {
		return (float)(Boolean.compare(this.dimension == 2, false)*(Math.PI)/2.0)+Projection.passAngles()[2] 
				+ (float)(Boolean.compare(this.dimension > 3, false))*(Projection.passAngles()[this.dimension-1]-Projection.passAngles()[2]);
	}
	
	public Vector3f setOrigin() {
		if (this.dimension <= 3) return new Vector3f(0,0,0);
		else return new Vector3f(-2,-2,0);
	}

	public float getDimension() {
		return this.dimension;
	}
	
}
