package assets;

import org.lwjgl.input.Keyboard;
import org.lwjgl.input.Mouse;
import org.lwjgl.util.vector.Vector3f;

import datasets.Dataset;
import datasets.Projection;

public class ViewControl {
	
	private static final float FORWARD_SPEED = 0.05f;
	private static final float SIDEWARD_SPEED = 0.05f;
	
	private float currentForwardSpeed = 0;
	private float currentSidewardSpeed = 0;
	private float currentVerticalSpeed = 0;
	
	
	private Vector3f position = new Vector3f(0f,0.2f,3.2f);
	private float pitch = 15;
	private float yaw = 0;
	private float scaledYaw = 0;
	
	public void move(Dataset data) {
		checkInputs(data);
		position.z += currentForwardSpeed*Math.cos(Math.toRadians(scaledYaw)) + currentSidewardSpeed*Math.sin(Math.toRadians(scaledYaw));
		position.x += -currentForwardSpeed*Math.sin(Math.toRadians(scaledYaw)) + currentSidewardSpeed*Math.cos(Math.toRadians(scaledYaw));
	}
	
	private void checkInputs(Dataset data) {
		if (Keyboard.isKeyDown(Keyboard.KEY_UP)) {
			this.currentForwardSpeed = -FORWARD_SPEED;
		}else if (Keyboard.isKeyDown(Keyboard.KEY_DOWN)) {
			this.currentForwardSpeed = FORWARD_SPEED;
		}else {
			this.currentForwardSpeed = 0;
		}
		
		if (Keyboard.isKeyDown(Keyboard.KEY_LEFT)) {
			this.currentSidewardSpeed = -SIDEWARD_SPEED;
		} else if (Keyboard.isKeyDown(Keyboard.KEY_RIGHT)) {
			this.currentSidewardSpeed = SIDEWARD_SPEED;
		} else {
			this.currentSidewardSpeed = 0;
		}
		
		if (Mouse.isInsideWindow() && Mouse.isButtonDown(0)) {
			yaw += Mouse.getDX();
			pitch -= Mouse.getDY();
			clampRotations();
		}
		
		if (Keyboard.isKeyDown(Keyboard.KEY_Q)) {
			Projection.manipulateProjectionMatrix(Keyboard.KEY_Q);
			data.update(Keyboard.KEY_Q);
		}
		
		if (Keyboard.isKeyDown(Keyboard.KEY_W)) {
			Projection.manipulateProjectionMatrix(Keyboard.KEY_W);
			data.update(Keyboard.KEY_W);
		}
		if (Keyboard.isKeyDown(Keyboard.KEY_E)) {
			Projection.manipulateProjectionMatrix(Keyboard.KEY_E);
			data.update(Keyboard.KEY_E);
		}
		if (Keyboard.isKeyDown(Keyboard.KEY_R)) {
			Projection.manipulateProjectionMatrix(Keyboard.KEY_R);
			data.update(Keyboard.KEY_R);
		}
		if (Keyboard.isKeyDown(Keyboard.KEY_T)) {
			Projection.manipulateProjectionMatrix(Keyboard.KEY_T);
			data.update(Keyboard.KEY_T);
		}
		if (Keyboard.isKeyDown(Keyboard.KEY_Y)) {
			Projection.manipulateProjectionMatrix(Keyboard.KEY_Y);
			data.update(Keyboard.KEY_Y);
		}
		if (Keyboard.isKeyDown(Keyboard.KEY_U)) {
			Projection.manipulateProjectionMatrix(Keyboard.KEY_U);
			data.update(Keyboard.KEY_U);
		}
		if (Keyboard.isKeyDown(Keyboard.KEY_I)) {
			Projection.manipulateProjectionMatrix(Keyboard.KEY_I);
			data.update(Keyboard.KEY_I);
		}
		if (Keyboard.isKeyDown(Keyboard.KEY_O)) {
			Projection.manipulateProjectionMatrix(Keyboard.KEY_O);
			data.update(Keyboard.KEY_O);
		}
		if (Keyboard.isKeyDown(Keyboard.KEY_P)) {
			Projection.manipulateProjectionMatrix(Keyboard.KEY_P);
			data.update(Keyboard.KEY_P);
		}
		if (Keyboard.isKeyDown(Keyboard.KEY_A)) {
			Projection.manipulateProjectionMatrix(Keyboard.KEY_A);
			data.update(Keyboard.KEY_Q);
		}
		
		if (Keyboard.isKeyDown(Keyboard.KEY_S)) {
			Projection.manipulateProjectionMatrix(Keyboard.KEY_S);
			data.update(Keyboard.KEY_W);
		}
		if (Keyboard.isKeyDown(Keyboard.KEY_D)) {
			Projection.manipulateProjectionMatrix(Keyboard.KEY_D);
			data.update(Keyboard.KEY_E);
		}
		if (Keyboard.isKeyDown(Keyboard.KEY_F)) {
			Projection.manipulateProjectionMatrix(Keyboard.KEY_F);
			data.update(Keyboard.KEY_R);
		}
		if (Keyboard.isKeyDown(Keyboard.KEY_G)) {
			Projection.manipulateProjectionMatrix(Keyboard.KEY_G);
			data.update(Keyboard.KEY_T);
		}
		if (Keyboard.isKeyDown(Keyboard.KEY_H)) {
			Projection.manipulateProjectionMatrix(Keyboard.KEY_H);
			data.update(Keyboard.KEY_Y);
		}
		if (Keyboard.isKeyDown(Keyboard.KEY_J)) {
			Projection.manipulateProjectionMatrix(Keyboard.KEY_J);
			data.update(Keyboard.KEY_U);
		}
		if (Keyboard.isKeyDown(Keyboard.KEY_K)) {
			Projection.manipulateProjectionMatrix(Keyboard.KEY_K);
			data.update(Keyboard.KEY_I);
		}
		if (Keyboard.isKeyDown(Keyboard.KEY_L)) {
			Projection.manipulateProjectionMatrix(Keyboard.KEY_L);
			data.update(Keyboard.KEY_O);
		}
		if (Keyboard.isKeyDown(Keyboard.KEY_SEMICOLON)) {					// Tilsvarer '' p norsk tastatur
			Projection.manipulateProjectionMatrix(Keyboard.KEY_SEMICOLON);
			data.update(Keyboard.KEY_P);
		}
	}
	

	
	
	private void clampRotations() {
		if (pitch < -45) {
			pitch = -45;
		} else if (pitch > 45) {
			pitch = 45;
		}
		
		scaledYaw =(float) (int) (yaw >= 0 ? yaw - 360f*((int)(yaw/360f)) : yaw + 360f*((int)((Math.abs(yaw)/360f)+1)));
	}
	


	public float getCurrentVerticalSpeed() {
		return currentVerticalSpeed;
	}

	public Vector3f getPosition() {
		return position;
	}

    
	public float getPitch() {
		return pitch;
	}

	public float getYaw() {
		return yaw;
	}

	public static float getForwardSpeed() {
		return FORWARD_SPEED;
	}

	public static float getSidewardSpeed() {
		return SIDEWARD_SPEED;
	}

	public float getCurrentForwardSpeed() {
		return currentForwardSpeed;
	}

	public float getCurrentSidewardSpeed() {
		return currentSidewardSpeed;
	}

	public float getScaledYaw() {
		return scaledYaw;
	}

	public void setHeight(float height) {
		position.y = height;
		
	}

	public void increasePosition(float x, float y, float z) {
		position.x += x;
		position.y += y;
		position.z += z;
	}

	
}
