package datasets;

import java.util.ArrayList;

import org.lwjgl.util.vector.Vector3f;

import assets.Point;
import assets.AxisVector;
import renderer.Loader;
import utilities.MNMatrix;

public class Dataset {

	public MNMatrix dataMatrix;
	private ArrayList<Point> points;
	private Projection projection;
	private Loader loader;
	private ArrayList<AxisVector> avs;
	private ArrayList<AxisVector> manipulatedAvs;
	
	public Dataset(MNMatrix matrix, Loader loader) {
		this.loader = loader;
		this.dataMatrix = matrix;
		
		points = new ArrayList<Point>();
		projection = new Projection(matrix.getRows(), matrix.getCols());
		MNMatrix projMatrix = projection.createProjectionMatrix();
		MNMatrix resultMatrix = new MNMatrix(3, matrix.getRows());
		ArrayList<Float> res = new ArrayList<Float>();
		matrix.transpose();
		
		for (int pr = 0; pr < 3; pr++) {	
			for (int c = 0; c < matrix.getCols(); c++) {
				float sum = 0;
				for (int r = 0; r < matrix.getRows(); r++) {
					sum += projMatrix.getValue(pr, r)*matrix.getValue(r, c);
				}
				res.add(sum);
				sum = 0;
			}
		}
		resultMatrix.fillMatrix(res);
		
		matrix.transpose();
		for (int c = 0; c < resultMatrix.getCols(); c++) {
			
				points.add(new Point(new Vector3f(resultMatrix.getValue(0, c), resultMatrix.getValue(1, c), resultMatrix.getValue(2, c)),loader));
		}
				
		avs = new ArrayList<AxisVector>();
		for (int dim = 0; dim < matrix.getCols(); dim++) {
			avs.add(new AxisVector(dim+1, matrix.getCols(), loader));
		}
		manipulatedAvs = avs;
	}
	
	public ArrayList<Point> getPoints() {
		return points;
	}

	public ArrayList<AxisVector> getManipAvs() {
		return manipulatedAvs;
	}
	
	public void update(int key) {
		float[] angles = Projection.passAngles();
		
		if ((key-16)>=angles.length) return;
		
		points = new ArrayList<Point>();
		
		MNMatrix projMatrix = projection.getMatrix();
		MNMatrix resultMatrix = new MNMatrix(3, dataMatrix.getRows());
		ArrayList<Float> res = new ArrayList<Float>();
		
		MNMatrix projection = Project(angles, key);
		
		dataMatrix.transpose();
		
		for (int pr = 0; pr < 3; pr++) {	
			for (int c = 0; c < projection.getCols(); c++) {
				float sum = 0;
				for (int r = 0; r < projection.getRows(); r++) {
					sum += projMatrix.getValue(pr, r)*projection.getValue(r, c);
				}
				res.add(sum);
				sum = 0;
			}
		}
		
		resultMatrix.fillMatrix(res);
		
		for (int c = 0; c < resultMatrix.getCols(); c++) {
			
				points.add(new Point(new Vector3f(resultMatrix.getValue(0, c), resultMatrix.getValue(1, c), resultMatrix.getValue(2, c)),loader));
		}
		
		manipulatedAvs = new ArrayList<AxisVector>();
		for (AxisVector av : avs) {
			res = new ArrayList<Float>();
			MNMatrix resVec = new MNMatrix(3, 1);
			
			for (int pr = 0; pr < 3; pr++) {	
				float sum = 0;
				for (int r = 0; r < dataMatrix.getRows(); r++) {
					sum += projMatrix.getValue(pr, r)*av.getVec().getValue(r, 0);
				}
				res.add(sum);
				sum = 0;
			}
			resVec.fillMatrix(res);
			AxisVector resAv = new AxisVector(av);
			resAv.setVec(resVec);
			manipulatedAvs.add(resAv);
		}
		dataMatrix.transpose();
		
	}
	
	public MNMatrix Project(float [] angles, int key) {
		int currentAxis = key-16;
		MNMatrix axisMatrix = new MNMatrix(dataMatrix.getRows(), angles.length);
		float currentWorkingAngle = angles[currentAxis];
		
		for (int r = 0; r < dataMatrix.getRows(); r++) 
			axisMatrix.setValue(r, currentAxis, 1);	

		MNMatrix normalizedMatrix = dataMatrix.normalizeRows();
		
		MNMatrix tildeB = normalizedMatrix.dotRows(axisMatrix);
		tildeB = normalizedMatrix.scalarMultiplyVector(tildeB);
		tildeB = axisMatrix.subtract(tildeB);
		MNMatrix markedB = tildeB.normalizeRows();
		
		MNMatrix cosine = normalizedMatrix. scalarMultiply((float)Math.cos(currentWorkingAngle));
		MNMatrix sine = markedB.scalarMultiply((float) Math.sin(currentWorkingAngle));
		cosine.add(sine);
		MNMatrix proje = cosine.scalarMultiplyVector(dataMatrix.getNorms());
		proje.transpose();
		
		return proje;
	}
}
