package utilities;

import java.util.ArrayList;

public class MNMatrix {

	private int rows;
	private int cols;
	
	public ArrayList<Float> matrix;
	
	public MNMatrix(int rows, int cols) {
		this.rows = rows;
		this.cols = cols;
		
		fillEmptyMatrix();
	}
	
	public void fillEmptyMatrix(){
		matrix = new ArrayList<Float>(rows*cols);
		for (int r = 0; r < rows; r++){
			for (int c = 0; c < cols; c++){
				matrix.add(0f);
			}
		}
	}
	
	public void fillMatrix(ArrayList<Float> values){
		matrix = new ArrayList<Float>(rows*cols);
		for (Float v : values) {
			matrix.add(v);
		}
	}
	
	public float getValue(int row, int col) {
		return matrix.get(col + row*this.cols);
	}
	
	public void setValue(int row, int col, float value) {
		matrix.set(col + row*this.cols, value);
	}

	public int getRows() {
		return rows;
	}

	public int getCols() {
		return cols;
	}
	
	public void print() {
		for (int r = 0; r < rows; r++){
			for (int c = 0; c < cols; c++) {
				System.out.print(matrix.get(c + r*this.cols) + " ");
			}
			System.out.println("");
		}
	}
	
	public void transpose() {
		MNMatrix temp = new MNMatrix(this.cols, this.rows);
		temp.fillEmptyMatrix();
		for (int c = 0; c < cols; c++) {
			for (int r = 0; r < rows; r++) {
				temp.setValue(c, r, this.getValue(r, c));
			}
		}
		this.matrix = temp.matrix;
		this.rows = temp.rows;
		this.cols = temp.cols;
 	}

	public void add(MNMatrix m) {
		if (rows != m.rows || cols != m.cols) {
			return;
		}
		
		for (int r = 0; r < rows; r++) {
			for (int c = 0; c < cols; c++) {
				setValue(r, c, this.getValue(r, c)+m.getValue(r, c));
			}
		}
	}
	
	public MNMatrix multiply(MNMatrix rhs) {
		if (this.getCols() != rhs.getRows()) return this;
		
		ArrayList<Float> res = new ArrayList<Float>();
		MNMatrix resMatrix = new MNMatrix(this.getRows(),rhs.getCols());

		for (int lhsr = 0; lhsr < this.getRows(); lhsr++) {	
			for (int c = 0; c < rhs.getCols(); c++) {
				float sum = 0;
				for (int r = 0; r < rhs.getRows(); r++) {
					sum += this.getValue(lhsr, r)*rhs.getValue(r, c);
				}
				res.add(sum);
			}
		}
		resMatrix.fillMatrix(res);
		return resMatrix;
	}
	
	public MNMatrix normalizeRows() {
		MNMatrix resMatrix = new MNMatrix(this.rows,this.cols);

		for (int point = 0; point < this.rows; point++) {
			float sum = 0;
			for (int dimension = 0; dimension < this.cols; dimension++) {
				sum += this.getValue(point, dimension)*this.getValue(point, dimension);
			}
			sum = (float)Math.sqrt(sum);
			for (int dimension = 0; dimension < this.cols; dimension++) {
				resMatrix.setValue(point, dimension, this.getValue(point, dimension)/sum);
			}
		}
		return resMatrix;	
	}
	
	public MNMatrix dotRows(MNMatrix rhs) {
		if (this.getCols() != rhs.getCols()) return null;
		MNMatrix resMatrix = new MNMatrix(this.rows,1);
		for (int row = 0; row < this.rows; row++) {
			float sum = 0;
			for (int elem = 0; elem < this.cols; elem++)
				sum += this.getValue(row, elem)*rhs.getValue(row, elem);
			resMatrix.setValue(row, 0, sum);
		}
		return resMatrix;
	}
	
	public MNMatrix scalarMultiply(float value) {
		MNMatrix resMatrix = new MNMatrix(this.rows, this.cols);
		for (int r = 0; r < this.rows; r++) {
			for (int c = 0; c < this.cols; c++) {
				resMatrix.setValue(r, c, this.getValue(r, c)*value);
			}
		}
		return resMatrix;
	}
	
	public MNMatrix scalarMultiplyVector(MNMatrix lhs) {
		if (lhs.getCols() != 1) return null;
		MNMatrix resMatrix = new MNMatrix(this.rows, this.cols);
		
		for (int r = 0; r < this.rows; r++) {
			for (int c = 0; c < this.cols; c++) {
				resMatrix.setValue(r, c, this.getValue(r, c)*lhs.getValue(r, 0));
			}
		}
		
		return resMatrix;
	}
	
	public MNMatrix subtract(MNMatrix rhs) {
		if (this.rows != rhs.rows || this.cols != rhs.cols) return null;
		
		MNMatrix resMatrix = new MNMatrix(this.rows, this.cols);
		for (int r = 0; r < this.rows; r++) {
			for (int c = 0; c < this.cols; c++) {
				resMatrix.setValue(r, c, this.getValue(r, c)-rhs.getValue(r, c));
			}
		}
		return resMatrix;
	}
	
	public MNMatrix getNorms() {
		MNMatrix resMatrix = new MNMatrix(this.rows,1);
		for (int r = 0; r < this.rows; r++) {
			float sum = 0;
			for (int c = 0; c < this.cols; c++) {
				sum += this.getValue(r, c)*this.getValue(r, c);
			}
			resMatrix.setValue(r, 0, (float)Math.sqrt(sum));
		}
		return resMatrix;
	}
}
