package utilities;

import org.lwjgl.opengl.Display;
import org.lwjgl.util.vector.Matrix4f;
import org.lwjgl.util.vector.Vector3f;

import assets.ViewControl;


public class Utils {
	
	public static Matrix4f createProjectionMatrix(float FOV, float zNear, float zFar, int width, int height) {
		Matrix4f projectionMatrix = new Matrix4f();
		
		float aspectRatio = (float) Display.getWidth() / (float) Display.getHeight();
		float yComp = (float) (1f / Math.tan(Math.toRadians(FOV / 2f)));
		float xComp = yComp / aspectRatio;
		projectionMatrix.m00 = xComp;
		projectionMatrix.m11 = yComp;
		projectionMatrix.m22 = -((zFar + zNear) / (zFar - zNear));
		projectionMatrix.m23 = -1;
		projectionMatrix.m32 = -((2 * zNear * zFar) / (zFar - zNear));
		projectionMatrix.m33 = 0;
		return projectionMatrix;
	}
	
	public static Matrix4f createModelMatrix(Vector3f translation, float rx, float ry, float rz, float scale) {
		Matrix4f matrix = new Matrix4f();
		Matrix4f.translate(translation, matrix, matrix);
		Matrix4f.rotate((float)Math.toRadians(rx), new Vector3f(1,0,0), matrix, matrix);
		Matrix4f.rotate((float)Math.toRadians(ry), new Vector3f(0,1,0), matrix, matrix);
		Matrix4f.rotate((float)Math.toRadians(rz), new Vector3f(0,0,1), matrix, matrix);
		Matrix4f.scale(new Vector3f(scale,scale,scale), matrix, matrix);
		return matrix;
	}
	
	public static Matrix4f createViewMatrix(ViewControl camera) {
		Matrix4f viewMatrix = new Matrix4f();
	
		Matrix4f.rotate((float) Math.toRadians(camera.getPitch()), new Vector3f(1,0,0),viewMatrix, viewMatrix);
		Matrix4f.rotate((float) Math.toRadians(camera.getYaw()), new Vector3f(0,1,0), viewMatrix, viewMatrix);
	
		Vector3f cameraPos = camera.getPosition();
		Vector3f negatedCameraPos = new Vector3f(-cameraPos.x,-cameraPos.y, -cameraPos.z);
		Matrix4f.translate(negatedCameraPos, viewMatrix, viewMatrix);
		
		
		return viewMatrix;
	}
	
}
